@@ -25,12 +25,13 @@
 #include "ignition/transport/Node.hh"
 #include "ignition/transport/TopicUtils.hh"
 #include "ignition/transport/test_config.h"
-#include "msg/int.pb.h"
+#include "msgs/int.pb.h"
 
 using namespace ignition;
 
 std::string partition;
 std::string topic = "/foo";
+std::mutex exitMutex;
 
 int data = 5;
 bool cbExecuted;
@@ -192,7 +193,8 @@ void CreateSubscriber()
 /// \brief Use two threads using their own transport nodes. One thread
 /// will publish a message, whereas the other thread is subscribed to the topic.
 /// \param[in] _scope Scope used to advertise the topic.
-void CreatePubSubTwoThreads(const transport::Scope &_sc = transport::Scope::All)
+void CreatePubSubTwoThreads(
+  const transport::Scope_t &_sc = transport::Scope_t::All)
 {
   reset();
 
@@ -231,7 +233,7 @@ TEST(NodeTest, PubWithoutAdvertise)
   // Check that an invalid namespace is ignored. The callbacks are expecting an
   // empty namespace.
   transport::Node node1(partition, "invalid namespace");
-  transport::Node node2;
+  transport::Node node2(partition, "");
 
   // Check the advertised/subscribed topics and advertised services.
   EXPECT_TRUE(node1.AdvertisedTopics().empty());
@@ -450,7 +452,7 @@ TEST(NodeTest, ClassMemberCallback)
 /// advertising a topic with "Process" scope.
 TEST(NodeTest, ScopeProcess)
 {
-  CreatePubSubTwoThreads(transport::Scope::Process);
+  CreatePubSubTwoThreads(transport::Scope_t::Process);
 }
 
 //////////////////////////////////////////////////
@@ -458,7 +460,7 @@ TEST(NodeTest, ScopeProcess)
 /// advertising a topic with "Host" scope.
 TEST(NodeTest, ScopeHost)
 {
-  CreatePubSubTwoThreads(transport::Scope::Host);
+  CreatePubSubTwoThreads(transport::Scope_t::Host);
 }
 
 //////////////////////////////////////////////////
@@ -466,7 +468,7 @@ TEST(NodeTest, ScopeHost)
 /// advertising a topic with "All" scope.
 TEST(NodeTest, ScopeAll)
 {
-  CreatePubSubTwoThreads(transport::Scope::All);
+  CreatePubSubTwoThreads(transport::Scope_t::All);
 }
 
 //////////////////////////////////////////////////
@@ -638,7 +640,7 @@ TEST(NodeTest, ServiceCallSyncTimeout)
     std::chrono::duration_cast<std::chrono::milliseconds>(t2 - t1).count();
 
   // Check if the elapsed time was close to the timeout.
-  EXPECT_NEAR(elapsed, timeout, 5.0);
+  EXPECT_NEAR(elapsed, timeout, 10.0);
 
   // Check that the service call response was not executed.
   EXPECT_FALSE(executed);
@@ -658,18 +660,27 @@ void createInfinitePublisher()
   EXPECT_TRUE(node.Advertise(topic));
 
   auto i = 0;
-  while (!terminatePub)
+  while (true)
   {
     EXPECT_TRUE(node.Publish(topic, msg));
     ++i;
     std::this_thread::sleep_for(std::chrono::milliseconds(10));
+
+    {
+      std::lock_guard<std::mutex> lock(exitMutex);
+      if (terminatePub)
+        break;
+    }
   }
 
   EXPECT_LT(i, 200);
 }
 
+//////////////////////////////////////////////////
+/// \brief Capture SIGINT and SIGTERM and flag that we want to exit.
 void signal_handler(int _signal)
 {
+  std::lock_guard<std::mutex> lock(exitMutex);
   if (_signal == SIGINT || _signal == SIGTERM)
     terminatePub = true;
 }
@@ -685,9 +696,20 @@ TEST(NodeTest, SigIntTermination)
   std::signal(SIGINT, signal_handler);
 
   auto thread = std::thread(createInfinitePublisher);
+#ifdef _WIN32
+  thread.detach();
+#endif
+
   std::this_thread::sleep_for(std::chrono::milliseconds(500));
   std::raise(SIGINT);
-  thread.join();
+
+#ifndef _WIN32
+  if (thread.joinable())
+    thread.join();
+#else
+  WaitForSingleObject(thread.native_handle(), INFINITE);
+  CloseHandle(thread.native_handle());
+#endif
 }
 
 //////////////////////////////////////////////////
@@ -697,20 +719,31 @@ TEST(NodeTest, SigTermTermination)
 {
   reset();
 
-  // Install a signal handler for SIGINT.
+  // Install a signal handler for SIGTERM.
   std::signal(SIGTERM, signal_handler);
 
   auto thread = std::thread(createInfinitePublisher);
+#ifdef _WIN32
+  thread.detach();
+#endif
+
   std::this_thread::sleep_for(std::chrono::milliseconds(500));
-  std::raise(SIGINT);
-  thread.join();
+  std::raise(SIGTERM);
+
+#ifndef _WIN32
+  if (thread.joinable())
+    thread.join();
+#else
+  WaitForSingleObject(thread.native_handle(), INFINITE);
+  CloseHandle(thread.native_handle());
+#endif
 }
 
 //////////////////////////////////////////////////
 int main(int argc, char **argv)
 {
   // Get a random partition name.
-  partition = testing::getRandomPartition();
+  partition = testing::getRandomNumber();
 
   // Set the partition name for this process.
   setenv("IGN_PARTITION", partition.c_str(), 1);
